from colorama import Fore, Style, init

init(autoreset=True)  # 启用颜色


class Colors:
    def green(x):
        return Fore.GREEN + Style.BRIGHT + x + Style.RESET_ALL

    def red(x):
        return Fore.RED + Style.BRIGHT + x + Style.RESET_ALL

    def yellow(x):
        return Fore.YELLOW + Style.BRIGHT + x + Style.RESET_ALL



import os
import re
import argparse
import argcomplete
from pload.utils import Colors
from pload.cmds import (
    create_venv, remove_venv, set_venv,
    get_venvs, get_python_versions, rdvenv
)

support_cmds = ['new', 'init', 'rm', 'cp', 'list']


def main():
    parser = argparse.ArgumentParser(
        description='A Minimal Python Venv Manage Tool. You can run '
                    f'{Colors.yellow("`pload [venv name]`")} to set '
                    f'{Colors.green("gloal venv")} and use '
                    f'{Colors.yellow("`activate`")} to activate the venv you set; and if there is '
                    f'{Colors.green(".venv")} under your path you can use '
                    f'{Colors.yellow("`pload .`")} to set and '
                    f'{Colors.yellow("`activate`")} to activate.',
        add_help=False
    )

    parser.add_argument('--help', '-h', action='store_true', help='Help.')
    args, all = parser.parse_known_args()

    subparsers = parser.add_subparsers(dest='command', title='commands')

    # new 子命令
    new_subparser = subparsers.add_parser(
        'new',
        help=f'Create a new virtual env under {Colors.green("global")} venv floder.'
    )
    new_subparser.add_argument(
        '--message', '-m',
        help='Venv message, the global name will be: '
             f'{Colors.yellow("<version>")}-{Colors.yellow("<message>")}, e.g. '
             f'{Colors.yellow("`pload new -m myvenv -v 3.8.10`")} get: '
             f'{Colors.green("3.8.10-myenv")}',
        default='normal'
    )
    new_subparser.add_argument(
        '--version', '-v',
        help=f'{Colors.green("Python version")} this venv uses.'
    )
    new_subparser.add_argument(
        '--channel', '-c',
        help='Channel while downloading requirements.'
    )
    new_subparser.add_argument(
        '--requirements', '-r', nargs='+',
        help=f'{Colors.green("Requirements")} needed to install for this venv. e.g. '
        f'{Colors.yellow("`pload new -m myvenv -v 3.8.10 -r numpy matplotlib pandas`")}'
    )

    # init 子命令
    init_subparser = subparsers.add_parser(
        'init',
        help=f'Create a new virtual env under {Colors.green("current")} folder.'
    )
    init_subparser.add_argument(
        '--version', '-v',
        help='python version this venv uses.'
    )
    init_subparser.add_argument(
        '--channel', '-c',
        help='Channel for downloading requirements.'
    )
    init_subparser.add_argument(
        '--requirements', '-r', nargs='+',
        help='Requirements needed for this venv.'
    )

    # rm 子命令
    rm_subparser = subparsers.add_parser('rm', help='Remove virtual envs.')
    rm_subparser.add_argument(
        '--envs', '-n', nargs='+',
        help='envs to remove'
    )
    rm_subparser.add_argument(
        '--expression', '-re',
        help=f'use {Colors.green("regulation expression")} to remove venvs.'
    )

    # cp 子命令
    cp_subparser = subparsers.add_parser('cp', help='Copy one venv to another.')
    cp_subparser.add_argument(
        '--from', '-f',
        help='from which venv name'
    )
    cp_subparser.add_argument(
        '--to', '-t',
        help="copy to which venv, if use '.' represents for current folder."
    )

    # list 子命令
    list_subparser = subparsers.add_parser(
        'list',
        help='List global venvs & python versions(-v), support regulation expression.'
    )
    list_subparser.add_argument(
        '--expression', '-re',
        help=f'{Colors.green("regulation expression")} for searching.',
        default='.*'
    )
    list_subparser.add_argument(
        '--version', '-v',
        help='list python versions',
        action='store_true'
    )

    if len(all) == 0:
        cmds = '{' + str(support_cmds).strip('[]').replace("'", "") + '}'
        parser.usage = f"{parser.prog} [venv name] [-h] {cmds} ..."
        parser.print_help()
        exit(0)

    if all[0] not in support_cmds:
        if len(all) == 1:
            env_name = all[0]
            set_venv(env_name)
            exit(0)
        else:
            print(f'[!] you can only activate {Colors.red("one venv once")}. But get: {all}')
            exit(1)

    argcomplete.autocomplete(parser)  # 添加补全
    args = parser.parse_known_args()[0]

    if all[0] == 'new':
        message = args.message
        version = args.version
        requirements = args.requirements
        channel = args.channel

        if version is None or args.help:
            print(f'[!] {Colors.red("version")} is must.')
            print()
            new_subparser.print_help()
            exit(1)

        create_venv(
            version=version,
            requirements=requirements,
            channel=channel,
            message=message,
            is_local=False
        )
        return

    elif all[0] == 'init':
        version = args.version
        requirements = args.requirements
        channel = args.channel

        if version is None:
            print(f'[!] {Colors.red("version")} is must.')
            print()
            init_subparser.print_help()
            exit(1)

        create_venv(
            version=version,
            requirements=requirements,
            channel=channel,
            is_local=True
        )
        return

    elif all[0] == 'rm':
        envs = args.envs if args.envs else []
        expr = args.expression if args.expression else '^$'
        venvs = get_venvs()

        envs += [x for x in venvs if re.fullmatch(expr, x)]

        if len(envs) == 0:
            print(f'[!] {Colors.red("venvs to remove is empty!")}')
            print()
            rm_subparser.print_help()
            exit(1)

        for env in envs:
            if env == '.':
                if not os.path.exists(os.path.join(os.getcwd(), '.venv')):
                    print('[!] There are not local venv.')
                    exit(1)

                remove_venv('.')
                print(f'[*] .venv -> {Colors.green(os.path.join(os.getcwd(), ".venv"))} is removed.')
            elif env in venvs:
                user_input = input(f'please input {Colors.green(env)} to remove it: ')
                if user_input == env:
                    remove_venv(env)
                else:
                    print(f'[!] input and {Colors.green(env)} is not match')
                    exit(1)
            else:
                print(f'[!] {env} is not in global envs')
                exit(1)

            print(f'[*] {Colors.green(env)} is successfully uninstalled.')
        return

    elif all[0] == 'cp':
        print('TODO')
        return

    elif all[0] == 'list':
        if args.version is True:
            print(f'[*] {Colors.green("Python versions")} in pyenv:')
            print()

            expr = args.expression
            for v in get_python_versions():
                if re.fullmatch(expr, v):
                    print(f'    {v}')
            return

        expr = args.expression
        CUR = rdvenv('CUR')

        print(f'[*] {Colors.green("Virtual envs")} managed by pload:')
        print()

        if CUR is not None:
            if CUR[0] == '.':
                print(f' >  {Colors.yellow(CUR)}')

            if os.path.exists(os.path.join(os.getcwd(), '.venv')):
                if CUR[9:] != os.path.join(os.getcwd(), '.venv'):
                    print(f'    local -> {os.path.join(os.getcwd(), ".venv")}')

        for venv in get_venvs():
            if venv == CUR:
                print(f' >  {Colors.yellow(CUR)}')
            elif re.fullmatch(expr, venv):
                print(f'    {venv}')
        return


if __name__ == '__main__':
    main()



import os
import shutil
import sys
import re
import subprocess
from pathlib import Path
from pload.utils import Colors

venv_path = os.path.join(os.path.expanduser("~"), 'venvs')

'''platform: windows, linux'''
if sys.platform == 'win32':
    pyenv_path = os.environ.get('PYENV_HOME')
    pyenv_exe = os.path.join(pyenv_path, 'bin', 'pyenv.bat')
    pyenv_versions = os.path.join(pyenv_path, 'versions')
elif sys.platform == 'linux':
    # home = os.environ.get('HOME')
    home = os.path.expanduser("~")
    pyenv_path = os.path.join(home, '.pyenv')
    pyenv_exe = os.path.join(pyenv_path, 'bin', 'pyenv')
    pyenv_versions = os.path.join(pyenv_path, 'versions')


def install_python(version):
    print(f'[*] Downloading python v{version}')

    command = [pyenv_exe, "install", version]

    process = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)

    for line in iter(process.stdout.readline, ''):
        print(line, end='')

    for line in iter(process.stderr.readline, ''):
        print(line, end='')


def get_python_versions():
    path = Path(pyenv_versions)
    versions = [f.name for f in path.iterdir() if f.is_dir()]

    return versions


def get_venvs():
    if not os.path.exists(venv_path):
        return None

    venvs = [name for name in os.listdir(venv_path) if os.path.isdir(os.path.join(venv_path, name))]

    if 'scripts' in venvs:
        venvs.remove('scripts')

    return venvs


def set_venv(venv_name='.'):

    is_local = True if venv_name == '.' else False

    if sys.platform == 'win32':
        if is_local:
            where = os.path.join(os.getcwd(), '.venv')

            if not os.path.exists(where):
                print('[!] local venv is not created, please create it first.')
                exit(1)

            wrvenv('CUR', f'.venv -> {where}')
        else:
            where = os.path.join(venv_path, venv_name)

            if venv_name not in get_venvs():
                print(f'[!] venv: "{venv_name}" is not created, please create it first.')
                exit(1)

            wrvenv('CUR', venv_name)
    elif sys.platform == 'linux':
        if is_local:
            where = os.path.join(os.getcwd(), '.venv')

            if not os.path.exists(where):
                print('[!] local venv is not created, please create it first.')
                exit(1)

            wrvenv('CUR', f'.venv -> {where}')
        else:
            where = os.path.join(venv_path, venv_name)

            if venv_name not in get_venvs():
                print(f'[!] venv: "{venv_name}" is not created, please create it first.')
                exit(1)

            wrvenv('CUR', venv_name)


def remove_venv(venv_name):
    CUR = rdvenv('CUR')
    if venv_name == CUR or venv_name == CUR[0]:
        print(f'[!] Can not remove {Colors.red(CUR)} because it is under using.')
        exit(1)

    is_local = True if venv_name == '.' else False

    if sys.platform == 'win32':
        if is_local:
            where = os.path.join(os.getcwd(), '.venv')

            if not os.path.exists(where):
                print('[!] local venv is not created, can not remove.')
                exit(1)

            shutil.rmtree(where)
        else:
            where = os.path.join(venv_path, venv_name)

            if venv_name not in get_venvs():
                print(f'[!] venv: "{venv_name}" is not created, can not remove.')
                exit(1)

            shutil.rmtree(where)
    elif sys.platform == 'linux':
        if is_local:
            where = os.path.join(os.getcwd(), '.venv')

            if not os.path.exists(where):
                print('[!] local venv is not created, can not remove.')
                exit(1)

            shutil.rmtree(where)
        else:
            where = os.path.join(venv_path, venv_name)

            if venv_name not in get_venvs():
                print(f'[!] venv: "{venv_name}" is not created, can not remove.')
                exit(1)

            shutil.rmtree(where)


def create_venv(version, requirements=None, channel=None, message='normal', is_local=False):
    if version not in get_python_versions():
        choice = input(f'[*] version v{version} not installed, do you want to download it? [Y/N]: ')
        if choice == 'Y':
            install_python(version)
        else:
            print('[!] Failed while creating venv ... 😭')
            exit(1)

    if not is_local:  # not local, check message
        trimed_message = message.replace(' ', '_')

        if sys.platform == 'win32':
            if not re.compile(r"^(?!CON|PRN|AUX|NUL)[a-zA-Z0-9_-]+$").match(trimed_message):
                print(f'[!] message: "{message}" does not conform to windows folder naming convention.')
                exit(1)
        elif sys.platform == 'linux':
            if not re.compile(r"^[a-zA-Z0-9_-]+$").match(trimed_message):
                print(f'[!] message: "{message}" does not conform to Linux folder naming convention.')
                exit(1)

    '''process of create venv.'''
    if sys.platform == 'win32':
        python_exe = os.path.join(pyenv_versions, version, 'python.exe')
    elif sys.platform == 'linux':
        python_exe = os.path.join(pyenv_versions, version, 'bin', 'python')

    if is_local:  # check exsits
        target_path = os.path.join(os.getcwd(), '.venv')
        if os.path.exists(target_path):
            print('[!] local .venv is already exsits, please do not create duplicates.')
            exit(0)
        else:
            print(f'[*] Creating env: {Colors.green(".venv")} -> {Colors.green(target_path)}')
    else:
        target_path = os.path.join(venv_path, f'{version}-{trimed_message}')
        if get_venvs() is not None and f'{version}-{trimed_message}' in get_venvs():
            print(f'[!] global venv "{target_path}" is already exsits, please do not create duplicates.')
            exit(0)
        else:
            print(f'[*] Creating env: {Colors.green(version + "-" + trimed_message)} -> {Colors.green(target_path)}')

    command = [python_exe, '-m', 'venv', target_path]
    # print(f'create: {command}')
    create_process = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)

    create_process.wait()

    '''process of install packages.'''
    if requirements is not None and len(requirements) > 0:
        req = ''
        for s in requirements:
            req += Colors.green(s) + " "

        ch = f"-i {channel}" if channel is not None else ""
        print(f'[*] pip install {req} {ch}')
        # print(requirements)

        if sys.platform == 'win32':
            pip_exe = os.path.join(target_path, 'Scripts', 'pip.exe')
        elif sys.platform == 'linux':
            pip_exe = os.path.join(target_path, 'bin', 'pip')

        command = [pip_exe, 'install', *requirements] + (['-i', channel] if channel is not None else [])
        # print(f'pip: {command}')

        package_process = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
        for line in iter(package_process.stdout.readline, ''):
            print(line, end='')
        for line in iter(package_process.stderr.readline, ''):
            print(line, end='')

    if create_process.returncode == 0:
        if is_local:
            print()
            print(f'[*] Successfully create {Colors.green("local venv")}. 🌟')
        else:
            print()
            print(f'[*] Successfully create env: {Colors.green(version + "-" + trimed_message)} -> {Colors.green(target_path)}. 🌟 ')
    else:
        print()
        print(f'[!] {Colors.red("Failed")} while creating venv ... 😭')
        exit(1)


def wrvenv(key, value):
    file_path = os.path.join(venv_path, 'env_value')
    if not os.path.exists(file_path):
        with open(file_path, 'w') as f:
            pass

    existing_content = {}
    if os.path.getsize(file_path) > 0:
        with open(file_path, 'r') as f:
            for line in f:
                line = line.strip()
                if line and not line.startswith('#'):
                    k, v = line.split('=', 1)
                    existing_content[k] = v

    existing_content[key] = value

    with open(file_path, 'w') as f:
        for k, v in existing_content.items():
            f.write(f"{k}={v}\n")


def rdvenv(key):
    file_path = os.path.join(venv_path, 'env_value')
    if not os.path.exists(file_path):
        return None

    with open(file_path, 'r') as f:
        for line in f:
            line = line.strip()
            if line and not line.startswith('#'):
                k, v = line.split('=', 1)
                if k == key:
                    return v
    return None


if __name__ == '__main__':
    # install_python("3.")
    print(get_python_versions())
    # create_venv('3.9.8', message='torch', is_local=True)
    # set_venv('.')
    # remove_venv('.')



import os
import shutil
from setuptools import setup, find_packages
from setuptools.command.install import install


class PostInstallCommand(install):
    def run(self):
        install.run(self)

        '''init some env vars'''
        home_dir = os.path.expanduser("~")

        self.script_names = ["pload.sh", "pload.zsh", "pload.ps1", "pload.fish"]
        self.shell_configs = {  # suffix : config file path
            "sh": os.path.join(home_dir, ".bashrc"),
            "zsh": os.path.join(home_dir, ".zshrc"),
            "fish": os.path.join(home_dir, ".config", "fish", "config.fish"),
            "ps1": os.path.join(
                home_dir,
                "Documents",
                "PowerShell",
                "Microsoft.PowerShell_profile.ps1",
            ),
        }

        '''make global venv dir venvs'''
        self.venvs_dir = os.path.join(home_dir, "venvs", "scripts")
        os.makedirs(self.venvs_dir, exist_ok=True)

        self.copy_scripts_to_venvs()
        self.add_to_shell_config()

    def copy_scripts_to_venvs(self):
        install_scripts_dir = self.install_scripts

        for script_name in self.script_names:
            src_path = os.path.join(install_scripts_dir, script_name)
            dst_path = os.path.join(self.venvs_dir, script_name)

            if os.path.exists(src_path):
                print(f">>> Copying {src_path} to {dst_path}")
                shutil.copy(src_path, dst_path)
            else:
                print(f">>> Script {src_path} not found. Skipping.")

    def add_to_shell_config(self):
        for shell, config_path in self.shell_configs.items():
            script_name = f"pload.{shell}"
            script_path = os.path.join(self.venvs_dir, script_name)

            if not os.path.exists(config_path):
                print(f'[!] Can not find config: {config_path}. Skipping')
                continue

            with open(config_path, "r") as f:
                if script_path in f.read():
                    print(f"[!] {script_name} already added to {config_path}. Skipping.")
                    continue

            with open(config_path, "a") as f:
                if shell == "fish":
                    f.write(f"\nsource {script_path}\n")
                elif shell == "ps1":
                    f.write(f'\n. "{script_path}"\n')
                else:
                    f.write(f'\nsource "{script_path}"\n')

            print(f"[*] Added {script_name} to {config_path}")


setup(
    name="pload",
    version="0.2.0",
    description="A simple command line tool for python virtual env management.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author="Yunming Hu",
    author_email="hugonelsonm3@gmail.com",
    license="MIT",
    python_requires=">=3.8",
    install_requires=[
        "colorama",
        "argcomplete",
    ],
    packages=find_packages(where="src"),  # 自动发现包
    package_dir={"": "src"},              # 包根目录为 src
    entry_points={
        "console_scripts": [
            "python_virtual_env_load = pload.cli:main",
        ],
    },
    scripts=[
        os.path.join("script", "pload.sh"),
        os.path.join("script", "pload.zsh"),
        os.path.join("script", "pload.ps1"),
        os.path.join("script", "pload.fish"),
    ],
    cmdclass={
        "install": PostInstallCommand
    },
)
